@@ -47,7 +47,7 @@ class GTM:
         grids[:, 1] = 2 * (grids[:, 1] - max_grids[1] / 2) / max_grids[1]
         return grids
 
-    def fit(self, input_dataset):
+    def fit(self, input_dataset, y=None):
         """
         Train the GTM map
                 
@@ -56,7 +56,8 @@ class GTM:
         input_dataset : numpy.array or pandas.DataFrame
              Training dataset for GTM.
              input_dataset must be autoscaled.
-        
+        y : None
+            Ignored.
         Returns
         ----------
         self : returns an instance of self.
@@ -173,7 +174,7 @@ class GTM:
         
         return means, modes
     
-    def fit_transform(self, x, mean_flag=True):
+    def fit_transform(self, x, y=None, mean_flag=True):
         """
         Fit GTM model and transform X
 
@@ -181,6 +182,8 @@ class GTM:
         ----------
         x : numpy.array or pandas.DataFrame
             input_dataset must be autoscaled.
+        y : None
+            Ignored.
         mean_flag ; boolean, default True
             If True, output is mean, and if False, output is mode
 
@@ -198,7 +201,7 @@ class GTM:
         else:
             return modes
     
-    def transform(self, x, mean_flag=True):
+    def transform(self, x, y=None, mean_flag=True):
         """
         Transform X using constructed GTM model
 
@@ -206,6 +209,8 @@ class GTM:
         ----------
         x : numpy.array or pandas.DataFrame
             input_dataset must be autoscaled.
+        y : None
+            Ignored.
         mean_flag ; boolen, default True
             If True, output is mean, and if False, output is mode
 
@@ -500,7 +505,7 @@ class GTM:
         y = np.ravel(dataset[:, numbers_of_output_variables])
         parameters_and_r2_cv = []
         all_calculation_numbers = len(candidates_of_shape_of_map) * len(candidates_of_shape_of_rbf_centers) * len(
-            candidates_of_variance_of_rbfs) * len(candidates_of_lambda_in_em_algorithm)
+            candidates_of_variance_of_rbfs) * len(candidates_of_lambda_in_em_algorithm) * len(reps)
         calculation_number = 0
         for shape_of_map_grid in candidates_of_shape_of_map:
             for shape_of_rbf_centers_grid in candidates_of_shape_of_rbf_centers:
@@ -539,12 +544,35 @@ class GTM:
 
         # optimized GTMR
         parameters_and_r2_cv = np.array(parameters_and_r2_cv)
-        optimized_hyperparameter_number = \
-            np.where(parameters_and_r2_cv[:, 5] == np.max(parameters_and_r2_cv[:, 5]))[0][0]
+        r2_cv = np.array(list(map(float, parameters_and_r2_cv[:, 5])))
+        optimized_hyperparameter_number = np.where(r2_cv == np.max(r2_cv))[0][0]
         self.shape_of_map = [int(parameters_and_r2_cv[optimized_hyperparameter_number, 0]),
                              int(parameters_and_r2_cv[optimized_hyperparameter_number, 0])]
         self.shape_of_rbf_centers = [int(parameters_and_r2_cv[optimized_hyperparameter_number, 1]),
                                      int(parameters_and_r2_cv[optimized_hyperparameter_number, 1])]
-        self.variance_of_rbfs = parameters_and_r2_cv[optimized_hyperparameter_number, 2]
-        self.lambda_in_em_algorithm = parameters_and_r2_cv[optimized_hyperparameter_number, 3]
+        self.variance_of_rbfs = float(parameters_and_r2_cv[optimized_hyperparameter_number, 2])
+        self.lambda_in_em_algorithm = float(parameters_and_r2_cv[optimized_hyperparameter_number, 3])
         self.rep = parameters_and_r2_cv[optimized_hyperparameter_number, 4]
+
+    def set_params(self, **parameters):
+        for parameter, value in parameters.items():
+            setattr(self, parameter, value)
+        return self
+
+    def get_params(self, deep=True):
+        """
+        Parameters
+        ----------
+        deep : Ignored. (for compatibility with sklearn)
+
+        Returns
+        ----------
+        self : returns an dictionary of parameters.
+        """
+        
+        params = {'shape_of_map' : self.shape_of_map,
+                  'shape_of_rbf_centers' : self.shape_of_rbf_centers,
+                  'variance_of_rbfs' : self.variance_of_rbfs,
+                  'lambda_in_em_algorithm' : self.lambda_in_em_algorithm,
+                   'number_of_iterations' : self.number_of_iterations}
+        return params
